@@ -97,7 +97,7 @@ parseNode n a = do
 parseData :: String         -- ^ The JSON message as received from Ganeti
           -> Result Request -- ^ A (possible valid) request
 parseData body = do
-  decoded <- fromJResult $ decodeStrict body
+  decoded <- fromJResult "Parsing input IAllocator message" (decodeStrict body)
   let obj = fromJSObject decoded
   -- request parser
   request <- liftM fromJSObject (fromObj "request" obj)
@@ -113,7 +113,7 @@ parseData body = do
   let (kti, il) = assignIndices iobj
   -- cluster tags
   ctags <- fromObj "cluster_tags" obj
-  (map_n, map_i, ptags, csf) <- mergeData [] [] [] (nl, il, ctags)
+  (map_n, map_i, ptags) <- mergeData [] [] [] (nl, il, ctags)
   optype <- fromObj "type" request
   rqtype <-
       case optype of
@@ -130,45 +130,41 @@ parseData body = do
               ridx <- lookupInstance kti rname
               req_nodes <- fromObj "required_nodes" request
               ex_nodes <- fromObj "relocate_from" request
-              let ex_nodes' = map (stripSuffix $ length csf) ex_nodes
-              ex_idex <- mapM (Container.findByName map_n) ex_nodes'
+              ex_idex <- mapM (Container.findByName map_n) ex_nodes
               return $ Relocate ridx req_nodes (map Node.idx ex_idex)
         "multi-evacuate" ->
             do
               ex_names <- fromObj "evac_nodes" request
-              let ex_names' = map (stripSuffix $ length csf) ex_names
-              ex_nodes <- mapM (Container.findByName map_n) ex_names'
+              ex_nodes <- mapM (Container.findByName map_n) ex_names
               let ex_ndx = map Node.idx ex_nodes
               return $ Evacuate ex_ndx
         other -> fail ("Invalid request type '" ++ other ++ "'")
-  return $ Request rqtype map_n map_i ptags csf
+  return $ Request rqtype map_n map_i ptags
 
 -- | Format the result
-formatRVal :: String -> RqType -> [Node.AllocElement] -> JSValue
-formatRVal _ _ [] = JSArray []
+formatRVal :: RqType -> [Node.AllocElement] -> JSValue
+formatRVal _ [] = JSArray []
 
-formatRVal csf (Evacuate _) elems =
-    let sols = map (\(_, inst, nl) ->
-                        let names = Instance.name inst : map Node.name nl
-                        in map (++ csf) names) elems
+formatRVal (Evacuate _) elems =
+    let sols = map (\(_, inst, nl) -> Instance.name inst : map Node.name nl)
+               elems
         jsols = map (JSArray . map (JSString . toJSString)) sols
     in JSArray jsols
 
-formatRVal csf _ elems =
+formatRVal _ elems =
     let (_, _, nodes) = head elems
-        nodes' = map ((++ csf) . Node.name) nodes
+        nodes' = map Node.name nodes
     in JSArray $ map (JSString . toJSString) nodes'
 
 -- | Formats the response into a valid IAllocator response message.
 formatResponse :: Bool     -- ^ Whether the request was successful
                -> String   -- ^ Information text
-               -> String   -- ^ Suffix for nodes and instances
                -> RqType   -- ^ Request type
                -> [Node.AllocElement] -- ^ The resulting allocations
                -> String   -- ^ The JSON-formatted message
-formatResponse success info csf rq elems =
+formatResponse success info rq elems =
     let
         e_success = ("success", JSBool success)
         e_info = ("info", JSString . toJSString $ info)
-        e_nodes = ("nodes", formatRVal csf rq elems)
+        e_nodes = ("nodes", formatRVal rq elems)
     in encodeStrict $ makeObj [e_success, e_info, e_nodes]
