@@ -33,7 +33,7 @@ module Ganeti.HTools.Node
     -- ** Finalization after data loading
     , buildPeers
     , setIdx
-    , setName
+    , setAlias
     , setOffline
     , setXmem
     , setFmem
@@ -41,13 +41,21 @@ module Ganeti.HTools.Node
     , setSec
     , setMdsk
     , setMcpu
+    -- * Tag maps
+    , addTags
+    , delTags
+    , rejectAddTags
     -- * Instance (re)location
     , removePri
     , removeSec
     , addPri
+    , addPriEx
     , addSec
+    , addSecEx
     -- * Stats
     , availDisk
+    , availMem
+    , availCpu
     , conflictingPrimaries
     -- * Formatting
     , defaultFields
@@ -58,7 +66,6 @@ module Ganeti.HTools.Node
     , AssocList
     , AllocElement
     , noSecondary
-    , noLimitInt
     ) where
 
 import Data.List
@@ -80,6 +87,7 @@ type TagMap = Map.Map String Int
 -- | The node type.
 data Node = Node
     { name     :: String    -- ^ The node name
+    , alias    :: String    -- ^ The shortened name (for display purposes)
     , tMem     :: Double    -- ^ Total memory (MiB)
     , nMem     :: Int       -- ^ Node memory (MiB)
     , fMem     :: Int       -- ^ Free memory (MiB)
@@ -116,8 +124,9 @@ data Node = Node
 instance T.Element Node where
     nameOf = name
     idxOf = idx
-    setName = setName
+    setAlias = setAlias
     setIdx = setIdx
+    allNames n = [name n, alias n]
 
 -- | A simple name for the int, node association list.
 type AssocList = [(T.Ndx, Node)]
@@ -133,14 +142,6 @@ type AllocElement = (List, Instance.Instance, [Node])
 noSecondary :: T.Ndx
 noSecondary = -1
 
--- | No limit value
-noLimit :: Double
-noLimit = -1
-
--- | No limit int value
-noLimitInt :: Int
-noLimitInt = -1
-
 -- * Helper functions
 
 -- | Add a tag to a tagmap
@@ -184,7 +185,8 @@ create :: String -> Double -> Int -> Int -> Double
        -> Int -> Double -> Bool -> Node
 create name_init mem_t_init mem_n_init mem_f_init
        dsk_t_init dsk_f_init cpu_t_init offline_init =
-    Node { name  = name_init
+    Node { name = name_init
+         , alias = name_init
          , tMem = mem_t_init
          , nMem = mem_n_init
          , fMem = mem_f_init
@@ -204,26 +206,34 @@ create name_init mem_t_init mem_n_init mem_f_init
          , pCpu = 0
          , offline = offline_init
          , xMem = 0
-         , mDsk = noLimit
-         , mCpu = noLimit
-         , loDsk = noLimitInt
-         , hiCpu = noLimitInt
+         , mDsk = T.defReservedDiskRatio
+         , mCpu = T.defVcpuRatio
+         , loDsk = mDskToloDsk T.defReservedDiskRatio dsk_t_init
+         , hiCpu = mCpuTohiCpu T.defVcpuRatio cpu_t_init
          , utilPool = T.baseUtil
          , utilLoad = T.zeroUtil
          , pTags = Map.empty
          }
 
+-- | Conversion formula from mDsk\/tDsk to loDsk
+mDskToloDsk :: Double -> Double -> Int
+mDskToloDsk mval tdsk = floor (mval * tdsk)
+
+-- | Conversion formula from mCpu\/tCpu to hiCpu
+mCpuTohiCpu :: Double -> Double -> Int
+mCpuTohiCpu mval tcpu = floor (mval * tcpu)
+
 -- | Changes the index.
 --
 -- This is used only during the building of the data structures.
 setIdx :: Node -> T.Ndx -> Node
 setIdx t i = t {idx = i}
 
--- | Changes the name.
+-- | Changes the alias.
 --
 -- This is used only during the building of the data structures.
-setName :: Node -> String -> Node
-setName t s = t {name = s}
+setAlias :: Node -> String -> Node
+setAlias t s = t { alias = s }
 
 -- | Sets the offline attribute.
 setOffline :: Node -> Bool -> Node
@@ -235,18 +245,11 @@ setXmem t val = t { xMem = val }
 
 -- | Sets the max disk usage ratio
 setMdsk :: Node -> Double -> Node
-setMdsk t val = t { mDsk = val,
-                    loDsk = if val == noLimit
-                             then noLimitInt
-                             else floor (val * tDsk t) }
+setMdsk t val = t { mDsk = val, loDsk = mDskToloDsk val (tDsk t) }
 
 -- | Sets the max cpu usage ratio
 setMcpu :: Node -> Double -> Node
-setMcpu t val = t { mCpu = val, hiCpu = hcpu }
-    where new_hcpu = floor (val * tCpu t)::Int
-          hcpu = if new_hcpu < 0
-                 then noLimitInt
-                 else new_hcpu
+setMcpu t val = t { mCpu = val, hiCpu = mCpuTohiCpu val (tCpu t) }
 
 -- | Computes the maximum reserved memory for peers from a peer map.
 computeMaxRes :: P.PeerMap -> P.Elem
@@ -321,7 +324,9 @@ removeSec t inst =
         old_peers = peers t
         old_peem = P.find pnode old_peers
         new_peem =  old_peem - Instance.mem inst
-        new_peers = P.add pnode new_peem old_peers
+        new_peers = if new_peem > 0
+                    then P.add pnode new_peem old_peers
+                    else P.remove pnode old_peers
         old_rmem = rMem t
         new_rmem = if old_peem < old_rmem
                    then old_rmem
@@ -336,9 +341,21 @@ removeSec t inst =
          , failN1 = new_failn1, rMem = new_rmem, pDsk = new_dp
          , pRem = new_prem, utilLoad = new_load }
 
--- | Adds a primary instance.
+-- | Adds a primary instance (basic version).
 addPri :: Node -> Instance.Instance -> T.OpResult Node
-addPri t inst =
+addPri = addPriEx False
+
+-- | Adds a primary instance (extended version).
+addPriEx :: Bool               -- ^ Whether to override the N+1 and
+                               -- other /soft/ checks, useful if we
+                               -- come from a worse status
+                               -- (e.g. offline)
+         -> Node               -- ^ The target node
+         -> Instance.Instance  -- ^ The instance to add
+         -> T.OpResult Node    -- ^ The result of the operation,
+                               -- either the new version of the node
+                               -- or a failure mode
+addPriEx force t inst =
     let iname = Instance.idx inst
         new_mem = fMem t - Instance.mem inst
         new_dsk = fDsk t - Instance.dsk inst
@@ -350,11 +367,13 @@ addPri t inst =
         new_load = utilLoad t `T.addUtil` Instance.util inst
         inst_tags = Instance.tags inst
         old_tags = pTags t
+        strict = not force
     in case () of
          _ | new_mem <= 0 -> T.OpFail T.FailMem
-           | new_dsk <= 0 || mDsk t > new_dp -> T.OpFail T.FailDisk
-           | new_failn1 && not (failN1 t) -> T.OpFail T.FailMem
-           | l_cpu >= 0 && l_cpu < new_pcpu -> T.OpFail T.FailCPU
+           | new_dsk <= 0 -> T.OpFail T.FailDisk
+           | mDsk t > new_dp && strict -> T.OpFail T.FailDisk
+           | new_failn1 && not (failN1 t) && strict -> T.OpFail T.FailMem
+           | l_cpu >= 0 && l_cpu < new_pcpu && strict -> T.OpFail T.FailCPU
            | rejectAddTags old_tags inst_tags -> T.OpFail T.FailTags
            | otherwise ->
                let new_plist = iname:pList t
@@ -366,9 +385,13 @@ addPri t inst =
                          , pTags = addTags old_tags inst_tags }
                in T.OpGood r
 
--- | Adds a secondary instance.
+-- | Adds a secondary instance (basic version).
 addSec :: Node -> Instance.Instance -> T.Ndx -> T.OpResult Node
-addSec t inst pdx =
+addSec = addSecEx False
+
+-- | Adds a secondary instance (extended version).
+addSecEx :: Bool -> Node -> Instance.Instance -> T.Ndx -> T.OpResult Node
+addSecEx force t inst pdx =
     let iname = Instance.idx inst
         old_peers = peers t
         old_mem = fMem t
@@ -382,10 +405,12 @@ addSec t inst pdx =
         old_load = utilLoad t
         new_load = old_load { T.dskWeight = T.dskWeight old_load +
                                             T.dskWeight (Instance.util inst) }
+        strict = not force
     in case () of
-         _ | new_dsk <= 0 || mDsk t > new_dp -> T.OpFail T.FailDisk
+         _ | new_dsk <= 0 -> T.OpFail T.FailDisk
+           | mDsk t > new_dp && strict -> T.OpFail T.FailDisk
            | Instance.mem inst >= old_mem -> T.OpFail T.FailMem
-           | new_failn1 && not (failN1 t) -> T.OpFail T.FailMem
+           | new_failn1 && not (failN1 t) && strict -> T.OpFail T.FailMem
            | otherwise ->
                let new_slist = iname:sList t
                    r = t { sList = new_slist, fDsk = new_dsk
@@ -401,19 +426,36 @@ availDisk :: Node -> Int
 availDisk t =
     let _f = fDsk t
         _l = loDsk t
-    in
-      if _l == noLimitInt
-      then _f
-      else if _f < _l
-           then 0
-           else _f - _l
+    in if _f < _l
+       then 0
+       else _f - _l
+
+-- | Computes the amount of available memory on a given node
+availMem :: Node -> Int
+availMem t =
+    let _f = fMem t
+        _l = rMem t
+    in if _f < _l
+       then 0
+       else _f - _l
+
+-- | Computes the amount of available memory on a given node
+availCpu :: Node -> Int
+availCpu t =
+    let _u = uCpu t
+        _l = hiCpu t
+    in if _l >= _u
+       then _l - _u
+       else 0
 
 -- * Display functions
 
 showField :: Node -> String -> String
 showField t field =
     case field of
-      "name" -> name t
+      "idx"  -> printf "%4d" $ idx t
+      "name" -> alias t
+      "fqdn" -> name t
       "status" -> if offline t then "-"
                   else if failN1 t then "*" else " "
       "tmem" -> printf "%5.0f" $ tMem t
@@ -427,8 +469,10 @@ showField t field =
       "fdsk" -> printf "%5d" $ fDsk t `div` 1024
       "tcpu" -> printf "%4.0f" $ tCpu t
       "ucpu" -> printf "%4d" $ uCpu t
-      "plist" -> printf "%3d" $ length (pList t)
-      "slist" -> printf "%3d" $ length (sList t)
+      "pcnt" -> printf "%3d" $ length (pList t)
+      "scnt" -> printf "%3d" $ length (sList t)
+      "plist" -> show $ pList t
+      "slist" -> show $ sList t
       "pfmem" -> printf "%6.4f" $ pMem t
       "pfdsk" -> printf "%6.4f" $ pDsk t
       "rcpu"  -> printf "%5.2f" $ pCpu t
@@ -438,7 +482,8 @@ showField t field =
       "nload" -> printf "%5.3f" uN
       "ptags" -> intercalate "," . map (\(k, v) -> printf "%s=%d" k v) .
                  Map.toList $ pTags t
-      _ -> printf "<unknown field>"
+      "peermap" -> show $ peers t
+      _ -> T.unknownField
     where
       T.DynUtil { T.cpuWeight = uC, T.memWeight = uM,
                   T.dskWeight = uD, T.netWeight = uN } = utilLoad t
@@ -448,7 +493,9 @@ showField t field =
 showHeader :: String -> (String, Bool)
 showHeader field =
     case field of
+      "idx" -> ("Index", True)
       "name" -> ("Name", False)
+      "fqdn" -> ("Name", False)
       "status" -> ("F", False)
       "tmem" -> ("t_mem", True)
       "nmem" -> ("n_mem", True)
@@ -461,8 +508,10 @@ showHeader field =
       "fdsk" -> ("f_dsk", True)
       "tcpu" -> ("pcpu", True)
       "ucpu" -> ("vcpu", True)
-      "plist" -> ("pri", True)
-      "slist" -> ("sec", True)
+      "pcnt" -> ("pcnt", True)
+      "scnt" -> ("scnt", True)
+      "plist" -> ("primaries", True)
+      "slist" -> ("secondaries", True)
       "pfmem" -> ("p_fmem", True)
       "pfdsk" -> ("p_fdsk", True)
       "rcpu"  -> ("r_cpu", True)
@@ -471,7 +520,8 @@ showHeader field =
       "dload" -> ("lDsk", True)
       "nload" -> ("lNet", True)
       "ptags" -> ("PrimaryTags", False)
-      _ -> ("<unknown field>", False)
+      "peermap" -> ("PeerMap", False)
+      _ -> (T.unknownField, False)
 
 -- | String converter for the node list functionality.
 list :: [String] -> Node -> [String]
@@ -481,6 +531,6 @@ list fields t = map (showField t) fields
 defaultFields :: [String]
 defaultFields =
     [ "status", "name", "tmem", "nmem", "imem", "xmem", "fmem"
-    , "rmem", "tdsk", "fdsk", "tcpu", "ucpu", "plist", "slist"
+    , "rmem", "tdsk", "fdsk", "tcpu", "ucpu", "pcnt", "scnt"
     , "pfmem", "pfdsk", "rcpu"
     , "cload", "mload", "dload", "nload" ]
