@@ -25,12 +25,12 @@ module Ganeti.HTools.Utils
     (
       debug
     , sepSplit
-    , fst3
     , varianceCoeff
     , commaJoin
     , readEitherString
     , loadJSArray
     , fromObj
+    , maybeFromObj
     , tryFromObj
     , fromJVal
     , asJSObject
@@ -41,6 +41,7 @@ module Ganeti.HTools.Utils
     , annotateResult
     ) where
 
+import Control.Monad (liftM)
 import Data.List
 import qualified Text.JSON as J
 import Text.Printf (printf)
@@ -71,10 +72,6 @@ sepSplit sep s
     where (x, xs) = break (== sep) s
           ys = drop 1 xs
 
--- | Simple version of 'fst' for a triple
-fst3 :: (a, b, c) -> a
-fst3 (a, _, _) = a
-
 -- * Mathematical functions
 
 -- Simple and slow statistical functions, please replace with better
@@ -93,9 +90,9 @@ varianceCoeff lst =
 -- * JSON-related functions
 
 -- | Converts a JSON Result into a monadic value.
-fromJResult :: Monad m => J.Result a -> m a
-fromJResult (J.Error x) = fail x
-fromJResult (J.Ok x) = return x
+fromJResult :: Monad m => String -> J.Result a -> m a
+fromJResult s (J.Error x) = fail (s ++ ": " ++ x)
+fromJResult _ (J.Ok x) = return x
 
 -- | Tries to read a string from a JSON value.
 --
@@ -108,15 +105,34 @@ readEitherString v =
       _ -> fail "Wrong JSON type"
 
 -- | Converts a JSON message into an array of JSON objects.
-loadJSArray :: (Monad m) => String -> m [J.JSObject J.JSValue]
-loadJSArray = fromJResult . J.decodeStrict
+loadJSArray :: (Monad m)
+               => String -- ^ Operation description (for error reporting)
+               -> String -- ^ Input message
+               -> m [J.JSObject J.JSValue]
+loadJSArray s = fromJResult s . J.decodeStrict
 
--- | Reads a the value of a key in a JSON object.
+-- | Reads the value of a key in a JSON object.
 fromObj :: (J.JSON a, Monad m) => String -> [(String, J.JSValue)] -> m a
 fromObj k o =
     case lookup k o of
       Nothing -> fail $ printf "key '%s' not found in %s" k (show o)
-      Just val -> fromJResult $ J.readJSON val
+      Just val -> fromKeyValue k val
+
+-- | Reads the value of an optional key in a JSON object.
+maybeFromObj :: (J.JSON a, Monad m) => String -> [(String, J.JSValue)]
+                -> m (Maybe a)
+maybeFromObj k o =
+    case lookup k o of
+      Nothing -> return Nothing
+      Just val -> liftM Just (fromKeyValue k val)
+
+-- | Reads a JValue, that originated from an object key
+fromKeyValue :: (J.JSON a, Monad m)
+              => String     -- ^ The key name
+              -> J.JSValue  -- ^ The value to read
+              -> m a
+fromKeyValue k val =
+  fromJResult (printf "key '%s', value '%s'" k (show val)) (J.readJSON val)
 
 -- | Annotate a Result with an ownership information
 annotateResult :: String -> Result a -> Result a
@@ -127,7 +143,7 @@ annotateResult _ v = v
 -- than fromObj
 tryFromObj :: (J.JSON a) =>
               String -> [(String, J.JSValue)] -> String -> Result a
-tryFromObj t o k = annotateResult (t ++ " key '" ++ k ++ "'") (fromObj k o)
+tryFromObj t o k = annotateResult t (fromObj k o)
 
 -- | Small wrapper over readJSON.
 fromJVal :: (Monad m, J.JSON a) => J.JSValue -> m a
