@@ -35,7 +35,6 @@ import System.IO
 import qualified System
 
 import Text.Printf (printf, hPrintf)
-import Text.JSON (showJSON)
 
 import qualified Ganeti.HTools.Container as Container
 import qualified Ganeti.HTools.Cluster as Cluster
@@ -48,7 +47,6 @@ import Ganeti.HTools.Utils
 import Ganeti.HTools.Types
 
 import qualified Ganeti.Luxi as L
-import qualified Ganeti.OpCodes as OpCodes
 import Ganeti.Jobs
 
 -- | Options list and functions
@@ -126,10 +124,6 @@ formatOneline ini_cv plc_len fin_cv =
     printf "%.8f %d %.8f %8.3f" ini_cv plc_len fin_cv
                (if fin_cv == 0 then 1 else ini_cv / fin_cv)
 
--- | Submits a list of jobs and waits for all to finish execution
-execJobs :: L.Client -> [[OpCodes.OpCode]] -> IO (Result [String])
-execJobs client = L.submitManyJobs client . showJSON
-
 -- | Polls a set of jobs at a fixed interval until all are finished
 -- one way or another
 waitForJobs :: L.Client -> [String] -> IO (Result [JobStatus])
@@ -137,7 +131,7 @@ waitForJobs client jids = do
   sts <- L.queryJobsStatus client jids
   case sts of
     Bad x -> return $ Bad x
-    Ok s -> if any (<= JobRunning) s
+    Ok s -> if any (<= JOB_STATUS_RUNNING) s
             then do
               -- TODO: replace hardcoded value with a better thing
               threadDelay (1000000 * 15)
@@ -146,21 +140,21 @@ waitForJobs client jids = do
 
 -- | Check that a set of job statuses is all success
 checkJobsStatus :: [JobStatus] -> Bool
-checkJobsStatus = all (== JobSuccess)
+checkJobsStatus = all (== JOB_STATUS_SUCCESS)
 
 -- | Execute an entire jobset
-execJobSet :: String -> String -> Node.List
+execJobSet :: String -> Node.List
            -> Instance.List -> [JobSet] -> IO ()
-execJobSet _      _   _  _  [] = return ()
-execJobSet master csf nl il (js:jss) = do
+execJobSet _      _  _  [] = return ()
+execJobSet master nl il (js:jss) = do
   -- map from jobset (htools list of positions) to [[opcodes]]
   let jobs = map (\(_, idx, move, _) ->
-                      Cluster.iMoveToJob csf nl il idx move) js
+                      Cluster.iMoveToJob nl il idx move) js
   let descr = map (\(_, idx, _, _) -> Container.nameOf il idx) js
   putStrLn $ "Executing jobset for instances " ++ commaJoin descr
   jrs <- bracket (L.getClient master) L.closeClient
          (\client -> do
-            jids <- execJobs client jobs
+            jids <- L.submitManyJobs client jobs
             case jids of
               Bad x -> return $ Bad x
               Ok x -> do
@@ -172,7 +166,7 @@ execJobSet master csf nl il (js:jss) = do
        hPutStrLn stderr $ "Cannot compute job status, aborting: " ++ show x
        return ()
      Ok x -> if checkJobsStatus x
-             then execJobSet master csf nl il jss
+             then execJobSet master nl il jss
              else do
                hPutStrLn stderr $ "Not all jobs completed successfully: " ++
                          show x
@@ -192,17 +186,20 @@ main = do
       verbose = optVerbose opts
       shownodes = optShowNodes opts
 
-  (fixed_nl, il, ctags, csf) <- loadExternalData opts
+  (fixed_nl, il, ctags) <- loadExternalData opts
 
   let offline_names = optOffline opts
       all_nodes = Container.elems fixed_nl
-      all_names = map Node.name all_nodes
+      all_names = concatMap allNames all_nodes
       offline_wrong = filter (`notElem` all_names) offline_names
       offline_indices = map Node.idx $
-                        filter (\n -> Node.name n `elem` offline_names)
+                        filter (\n ->
+                                 Node.name n `elem` offline_names ||
+                                 Node.alias n `elem` offline_names)
                                all_nodes
       m_cpu = optMcpu opts
       m_dsk = optMdsk opts
+      csf = commonSuffix fixed_nl il
 
   when (length offline_wrong > 0) $ do
          hPrintf stderr "Wrong node name(s) set as offline: %s\n"
@@ -268,8 +265,8 @@ main = do
                       printf "Initial score: %.8f\n" ini_cv)
 
   unless oneline $ putStrLn "Trying to minimize the CV..."
-  let imlen = Container.maxNameLen il
-      nmlen = Container.maxNameLen nl
+  let imlen = maximum . map (length . Instance.alias) $ Container.elems il
+      nmlen = maximum . map (length . Node.alias) $ Container.elems nl
 
   (fin_tbl, cmd_strs) <- iterateDepth ini_tbl (optMaxLength opts)
                          (optDiskMoves opts)
@@ -311,7 +308,7 @@ main = do
               Nothing -> do
                 hPutStrLn stderr "Execution of commands possible only on LUXI"
                 exitWith $ ExitFailure 1
-              Just master -> execJobSet master csf fin_nl il cmd_jobs)
+              Just master -> execJobSet master fin_nl il cmd_jobs)
 
   when (optShowInsts opts) $ do
          putStrLn ""
