@@ -30,9 +30,7 @@ module Main (main) where
 import Data.List
 import Data.Maybe (isJust, fromJust, fromMaybe)
 import Monad
-#ifdef NO_CURL
 import System (exitWith, ExitCode(..))
-#endif
 import System.IO
 import System.FilePath
 import qualified System
@@ -65,28 +63,27 @@ options =
     ]
 
 -- | Serialize a single node
-serializeNode :: String -> Node.Node -> String
-serializeNode csf node =
-    printf "%s|%.0f|%d|%d|%.0f|%d|%.0f|%c" (Node.name node ++ csf)
+serializeNode :: Node.Node -> String
+serializeNode node =
+    printf "%s|%.0f|%d|%d|%.0f|%d|%.0f|%c" (Node.name node)
                (Node.tMem node) (Node.nMem node) (Node.fMem node)
                (Node.tDsk node) (Node.fDsk node) (Node.tCpu node)
                (if Node.offline node then 'Y' else 'N')
 
 -- | Generate node file data from node objects
-serializeNodes :: String -> Node.List -> String
-serializeNodes csf =
-    unlines . map (serializeNode csf) . Container.elems
+serializeNodes :: Node.List -> String
+serializeNodes = unlines . map serializeNode . Container.elems
 
 -- | Serialize a single instance
-serializeInstance :: String -> Node.List -> Instance.Instance -> String
-serializeInstance csf nl inst =
+serializeInstance :: Node.List -> Instance.Instance -> String
+serializeInstance nl inst =
     let
-        iname = Instance.name inst ++ csf
-        pnode = Container.nameOf nl (Instance.pNode inst) ++ csf
+        iname = Instance.name inst
+        pnode = Container.nameOf nl (Instance.pNode inst)
         sidx = Instance.sNode inst
         snode = (if sidx == Node.noSecondary
                     then ""
-                    else Container.nameOf nl sidx ++ csf)
+                    else Container.nameOf nl sidx)
     in
       printf "%s|%d|%d|%d|%s|%s|%s|%s"
              iname (Instance.mem inst) (Instance.dsk inst)
@@ -94,9 +91,9 @@ serializeInstance csf nl inst =
              pnode snode (intercalate "," (Instance.tags inst))
 
 -- | Generate instance file data from instance objects
-serializeInstances :: String -> Node.List -> Instance.List -> String
-serializeInstances csf nl =
-    unlines . map (serializeInstance csf nl) . Container.elems
+serializeInstances :: Node.List -> Instance.List -> String
+serializeInstances nl =
+    unlines . map (serializeInstance nl) . Container.elems
 
 -- | Return a one-line summary of cluster state
 printCluster :: Node.List -> Instance.List
@@ -128,10 +125,10 @@ fixSlash = map (\x -> if x == '/' then '_' else x)
 processData :: Result (Node.AssocList, Instance.AssocList, [String])
             -> Result (Node.List, Instance.List, String)
 processData input_data = do
-  (nl, il, _, csf) <- input_data >>= Loader.mergeData [] [] []
+  (nl, il, _) <- input_data >>= Loader.mergeData [] [] []
   let (_, fix_nl) = Loader.checkData nl il
-  let ndata = serializeNodes csf nl
-      idata = serializeInstances csf nl il
+  let ndata = serializeNodes nl
+      idata = serializeInstances nl il
       adata = ndata ++ ['\n'] ++ idata
   return (fix_nl, il, adata)
 
@@ -140,9 +137,10 @@ writeData :: Int
           -> String
           -> Options
           -> Result (Node.List, Instance.List, String)
-          -> IO ()
+          -> IO Bool
 writeData _ name _ (Bad err) =
-    printf "\nError for %s: failed to load data. Details:\n%s\n" name err
+  printf "\nError for %s: failed to load data. Details:\n%s\n" name err >>
+  return False
 
 writeData nlen name opts (Ok (nl, il, adata)) = do
   printf "%-*s " nlen name :: IO ()
@@ -155,7 +153,7 @@ writeData nlen name opts (Ok (nl, il, adata)) = do
   when (isJust shownodes) $
        putStr $ Cluster.printNodes nl (fromJust shownodes)
   writeFile (oname <.> "data") adata
-
+  return True
 
 -- | Main function.
 main :: IO ()
@@ -177,14 +175,16 @@ main = do
          let lsock = fromMaybe defaultLuxiSocket (optLuxi opts)
          let name = local
          input_data <- Luxi.loadData lsock
-         writeData nlen name opts (processData input_data)
+         result <- writeData nlen name opts (processData input_data)
+         when (not result) $ exitWith $ ExitFailure 2
 
 #ifndef NO_CURL
-  mapM_ (\ name ->
-            do
-              input_data <- Rapi.loadData name
-              writeData nlen name opts (processData input_data)
-        ) clusters
+  results <- mapM (\ name ->
+                    do
+                      input_data <- Rapi.loadData name
+                      writeData nlen name opts (processData input_data)
+                  ) clusters
+  when (not $ all id results) $ exitWith (ExitFailure 2)
 #else
   when (not $ null clusters) $ do
     putStrLn "RAPI/curl backend disabled at compile time, cannot scan clusters"
